#!/usr/bin/python

## Description: A pythonic wrapper for Nucleosome position prediction tool called NuPoP_F. It can handle multiple sequence in a FASTA file. Further details can be found at http://nucleosome.stats.northwestern.edu/
## Written by Vipin T Sreedharan, Friedrich Miescher Laboratory of the Max Planck Society, Tuebingen Germany.
 
import os, re, sys, struct, time
from Bio import SeqIO

def __main__():
	stime = time.asctime( time.localtime(time.time()) )
	print '----------------------------------------'
	print 'NuPoP started on ' + stime  
	print '----------------------------------------'
	print'\nStep:1 Checking input parameters\n' 
	try:
		fasta_file = sys.argv[1]
		linker_len = sys.argv[2]
		order_mm = sys.argv[3]
		linker_up = sys.argv[4]
		organism = sys.argv[5]
		res_format = sys.argv[6]
		pred_file = sys.argv[7]
		pred_file_path = sys.argv[8]
	except:
		sys.stderr.write("Check input parameters:\n\t1. A valid FASTA file\n\t2. Linker DNA length (100-500)\n\t3. Order of HMM (1 or 4)\n\t4. Linker length distribution (0-5)\n\t")
		sys.stderr.write("5. Species (1 : Homo sapiens, 2 : Mus musculus, 3 : Rattus norvegicus, 4 : Danio rerio, 5 : Drosophila melanogaster, 6 : Caenorhabditis elegans, 7 : Saccharomyces cerevisiae, 8 : Candida albicans, 9 : Schizosaccharomyces pombe, 10 : Arabidopsis thaliana, 11 : Zea mays)\n\t")
		sys.stderr.write("6. Result format (wig or text)\n\t7. Prediction result file name\n\t8. Temporory path\n")
		sys.exit(-1)
	## species available for NuPoP program	
	species = {'1' : "Homo sapiens", '2' : "Mus musculus", '3' : "Rattus norvegicus", '4' : "Danio rerio", '5' : "Drosophila melanogaster", '6' : "Caenorhabditis elegans", '7' : "Saccharomyces cerevisiae", '8' : "Candida albicans", '9' : "Schizosaccharomyces pombe", '10' : "Arabidopsis thaliana", '11' : "Zea mays"}
	os.system("mkdir -p " + pred_file_path)
	## log message 	
	print "Reading FASTA file                      : " + fasta_file + "\n"
	print "Maximum length of linker DNA            : " + linker_len + "\n"
	print "Order of Markov Model                   : " + order_mm + "\n"
	print "Linker length distribution update times : " + linker_up + "\n"
	print "Species selected                        : " + species[organism] + "\n\n"
	pred_file_handle = open(pred_file, "w+")
	if res_format == 'wig': pred_file_handle.write("track type=wiggle_0 name=\"Nucleosome_Position_Prediction\" description=\"Genome source " + species[organism] + "\"\n")
	print 'Step:2 Processing FASTA file\n'
	fasta_handle = open(fasta_file, "rU")
	for record in SeqIO.parse(fasta_handle, "fasta"):
		print 'Sequence identifier: ' + record.id + '\n'
		print 'Sequence length: '+ str(len(record.seq)) + '\n'
		fasta_in = pred_file_path + "/" + record.id + ".fa"
		fasta_outhand = open(fasta_in, "w+")
		fasta_outhand.write(record.format("fasta"))
		fasta_outhand.close()
		# Npred result file 
		npred_out = pred_file_path + "/"+record.id
		# compile NPred program
		print 'Compiling NuPoP for chromosome/contig: ' + str(record.id)
		try:
			## TODO Please add the right path to Npred.f90, It can be found in nupop_0.1/.
			os.system("/usr/bin/gfortran Npred.f90 -w -o " + pred_file_path + "/" + record.id + "Npred")
		except:
			sys.stderr.write("Compile NPred failed. Please make sure the path of your gfortran compiler and Npred.f90 program, Cannot continue, Program terminating.....\n")
			os.system("rm -rf " + pred_file_path)
			sys.exit(-1)
		print 'Done.'
		# run NuPoP
		print 'Predicting Nucleosome positioning for chromosome/contig: ' + str(record.id)
		try:
			os.system(pred_file_path + "/" + record.id + "Npred " + fasta_in + " " + linker_len + " " + order_mm + " " + linker_up + " " + organism + " " + npred_out)
		except:
			sys.stdout.write("NuPoP execution failed for chromosome/contig " + str(record.id) + ".\n")
			os.system("rm -rf " + fasta_in)
			os.system("rm -rf " + pred_file_path + '/' + record.id + "Npred")
			continue
		print 'Done.'
		try:	
			npred_res_handle = open(npred_out, "r")
		except:
			sys.stderr.write("NuPoP prediction result file fail to open.")
			sys.exit(-1)
		print 'Writing result files'
		## create binary files with nucleosome occupancy score and feature position
		nos = open(pred_file_path + "/" + record.id + ".nos", "wb")
		pos = open(pred_file_path + "/" + record.id + ".pos", "wb")
		if res_format == 'text':
			pred_file_handle.write("Chromosome/Contig = " + record.id + "\n")
			for line in npred_res_handle:
				line = line.strip()
				if re.search(r'^\s*Position', line):
					pred_file_handle.write(line + "\n")
					continue
				pred_file_handle.write(line + "\n")
				parts = line.split("  ")
				parts[2] = parts[2].lstrip()
				nos.write(struct.pack('f', float(parts[2])))
				pos.write(struct.pack('i', int(parts[0])))
		elif res_format == 'wig':	
			pred_file_handle.write("fixedStep chrom=" + record.id + " start=1 step=1 span=1\n")
			for line in npred_res_handle:
				line = line.strip()
				if re.search(r'^\s*Position', line): continue
				parts = line.split("  ")
				parts[2] = parts[2].lstrip()
				pred_file_handle.write(parts[2] + "\n")
				nos.write(struct.pack('f', float(parts[2])))
				pos.write(struct.pack('i', int(parts[0])))
		npred_res_handle.close()
		nos.close()
		pos.close()
		# clean up the contig based fasta and npred result file 
		os.system("rm -rf " + fasta_in + " " + npred_out )
		os.system("rm -rf " + pred_file_path +'/'+ record.id + "Npred")
		print "     Saving feature postion to (" + pred_file_path + '/' + record.id + '.pos' + ")\n"
		print "     Saving nucleosome occupancy score to (" + pred_file_path + '/' + record.id + '.nos' + ")\n" 
		print 'Done.'
	fasta_handle.close()	
	pred_file_handle.close()
	etime = time.asctime( time.localtime(time.time()) )
	print '----------------------------------------'
	print 'NuPoP finished on ' + etime  
	print '----------------------------------------'

if __name__=='__main__':__main__()
